/*=============================================================================

Copyright 2008 Francois Laupretre (francois@tekwire.net)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
=============================================================================*/

/* The data exposed here is everything a program needs to use the logmanager
* library */

#ifndef __LOGMANAGER_H
#define __LOGMANAGER_H

#include <apr.h>
#include <apr_file_io.h>
#include <apr_time.h>

#if APR_HAVE_STRING_H
#include <string.h>
#endif

/*----------------------------------------------*/

#ifndef _WIN32
#define SYMLINK_SUPPORT
#define HARDLINK_SUPPORT
#endif

/*----------------------------------------------*/

#ifndef NOW
typedef apr_time_t TIMESTAMP;

#define NOW	(TIMESTAMP)0
#endif

/*----------------------------------------------*/
/* logmanager_write() flags */

#define LMGRW_CANNOT_ROTATE	0x1

/* LOGMANAGER option flags */

#define LMGR_ACTIVE_LINK	0x01	/* Maintain link to active log */
#define LMGR_BACKUP_LINKS	0x02	/* Maintain link to backup logs */
#define LMGR_HARD_LINKS		0x04	/* Create hard links (not symbolic) */
#define LMGR_IGNORE_EOL		0x08	/* Don't rotate on eol only */
#define LMGR_IGNORE_ENOSPC	0x10	/* ignore 'no more space' errors */
#define LMGR_PID_FILE		0x20	/* Maintain PID file */

/*----------------------------------------------*/

#define LOGMANAGER_API_VERSION	3

/*----------------------------------------------*/

typedef struct
	{
	unsigned int api_version;
	char *base_path;
	unsigned int flags;
	struct
		{
		char *type;
		char *level;
		} compress;
	apr_off_t file_maxsize;
	apr_off_t global_maxsize;
	unsigned int keep_count;
	apr_fileperms_t create_mode;
	char *debug_file;
	int debug_level;
	char *rotate_cmd;
	/* API version 1 stops here */
	TIMESTAMP rotate_delay;
	TIMESTAMP purge_delay;
	/* API version 2 stops here */
	char *log_path;
	/* API version 3 stops here */
	} LOGMANAGER_OPTIONS;

/*----------------------------------------------*/

#ifdef IN_LMGR_LIB

#include "../include/compress.h"
#include "../include/file.h"
#include "../include/array.h"
#include "../include/checksum.h"

typedef struct
	{
	char *path;
	char *link;
	TIMESTAMP start;
	TIMESTAMP end;
	apr_off_t size;  /* File (compressed) size */
	apr_off_t osize; /* Original (uncompressed) size */
	CHECKSUM sum;
	} LOGFILE;

typedef struct
	{
	char *base_dir;
	size_t base_dir_len; /* strlen(base_dir) */
	char *base_path;
	size_t base_path_len; /* strlen(base_path) */
	char *log_path;
	size_t log_path_len; /* strlen(log_path) */
	char *log_dir;
	char *status_path;
	char *pid_path;
	unsigned int flags;
	struct
		{
		COMPRESS_HANDLER *handler;
		void *private;
		unsigned long ratio;
		char *level;
		} compress;
	struct
		{
		OFILE *fp;
		LOGFILE *file;
		} active;
	struct
		{
		ARRAY(LOGFILE) files;
		apr_off_t size;
		} backup;
	apr_off_t file_maxsize;
	apr_off_t global_maxsize;
	unsigned int keep_count;
	TIMESTAMP rotate_delay;
	TIMESTAMP purge_delay;
	apr_fileperms_t create_mode;
	char *rotate_cmd;
	struct
		{
		char *buf;
		apr_off_t len;
		} eol_buffer;
	TIMESTAMP last_write_time;
	struct
		{
		char *path;
		OFILE *fp;
		int level;
		} debug;
	struct
		{
		int write_count;
		int write2_count;
		int write3_count;
		int flush_count;
		int link_count;
		int refresh_backup_links_count;
		int refresh_active_link_count;
		int new_active_file_count;
		int rotate_count;
		int remove_oldest_count;
		int dump_count;
		int sync_count;
		} stats;
	} _LOGMANAGER_STRUCT;

typedef _LOGMANAGER_STRUCT LOGMANAGER;

#else
typedef struct { int dummy; } LOGMANAGER; /* Opaque to client */
#endif

/*----------------------------------------------*/
/* Functions */

extern LOGMANAGER *new_logmanager(LOGMANAGER_OPTIONS *opts);
extern void logmanager_destroy(LOGMANAGER *mp);
extern void logmanager_open(LOGMANAGER *mp,TIMESTAMP t);
extern void logmanager_close(LOGMANAGER *mp);
extern void logmanager_write(LOGMANAGER *mp, const char *buf, apr_off_t size
	,unsigned int flags, TIMESTAMP t);
extern void logmanager_flush(LOGMANAGER *mp);
extern void logmanager_rotate(LOGMANAGER *mp,TIMESTAMP t);
extern char *logmanager_compression_list(void);
extern char *logmanager_version(void);
extern void logmanager_display_stats(LOGMANAGER *mp);

/*----------------------------------------------*/
#endif	/* __LOGMANAGER_H */
